#!/usr/bin/env python

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer


_undo_list = []
_redo_list = []

_on_add_undo = None
_on_add_undo_args = None

_on_undo = None
_on_undo_args = None
_on_redo = None
_on_redo_args = None

def set_add_undo_callback(func,*args):
    global _on_add_undo
    global _on_add_undo_args
    _on_add_undo = func
    _on_add_undo_args = args

def set_undo_callback(func,*args):
    global _on_undo
    global _on_undo_args
    _on_undo = func
    _on_undo_args = args

def set_redo_callback(func,*args):
    global _on_redo
    global _on_redo_args
    _on_redo = func
    _on_redo_args = args

def has_undo():
    return bool(_undo_list)

def has_redo():
    return bool(_redo_list)

def add_undo(undoObject):
    _undo_list.append(undoObject)
    while _redo_list:
        _redo_list.pop()
    
    if _on_add_undo != None:
        _on_add_undo(*_on_add_undo_args)

def undo():
    if has_undo():
        undoObject = _undo_list.pop()
        undoObject.undo()
        _redo_list.append(undoObject)

        if _on_undo:
            _on_undo(undoObject,*_on_undo_args)

    
def redo():
    if has_redo():
        undoObject = _redo_list.pop()
        undoObject.redo()
        _undo_list.append(undoObject)
        
        if _on_redo:
            _on_redo(undoObject,*_on_redo_args)


def clear():

    _undo_list = []
    _redo_list = []

    _on_add_undo = None
    _on_add_undo_args = None

    _on_undo = None
    _on_undo_args = None
    _on_redo = None
    _on_redo_args = None


class AttribUndo(object):
    
    def __init__(self,element,old_attrib):

        self.element = element
        self.undo_attrib = old_attrib
        self.redo_attrib = element.attrib.copy()

    def undo(self):
        self.element.attrib = self.undo_attrib

    def redo(self):
        self.element.attrib = self.redo_attrib

class ChildrenUndo(object):

    def __init__(self,element,child_list):

        self.element = element
        self.undo_child_list = child_list
        self.redo_child_list = [child for child in element]

    def undo(self):
        rem_list = [child for child in self.element]
        for child in rem_list:
            self.element.remove(child)

        for child in self.undo_child_list:
            self.element.append(child)

    def redo(self):
        rem_list = [child for child in self.element]
        for child in rem_list:
            self.element.remove(child)

        for child in self.redo_child_list:
            self.element.append(child)

class TypeUndo(object):

    def __init__(self,element,type,attrib,child_list):

        self.element = element
        self.undo_type = type
        self.undo_attrib = attrib
        self.undo_child_list = child_list
        
        self.redo_type = element.tag
        self.redo_attrib = element.attrib.copy()
        self.redo_child_list = [child for child in element]

    def undo(self):
        self.element.tag = self.undo_type
        self.element.attrib = self.undo_attrib

        rem_list = [child for child in self.element]
        for child in rem_list:
            self.element.remove(child)

        for child in self.undo_child_list:
            self.element.append(child)

    def redo(self):
        self.element.tag = self.redo_type
        self.element.attrib = self.redo_attrib

        rem_list = [child for child in self.element]
        for child in rem_list:
            self.element.remove(child)

        for child in self.redo_child_list:
            self.element.append(child)

class ButtonDrawopsUndo(object):
    
    def __init__(self,TE,win_type,focus,state,function,Bstate,old_attrib):
        
        frame_elem = TE.Get_Element(win_type,focus,state)
        self.fs_name = frame_elem.attrib['style']
        
        self.element = TE.Get_Element(win_type,focus,state,"",function,
                Bstate)
        
        self.TE = TE
        self.function = function
        self.Bstate = Bstate


        self.undo_attrib = old_attrib

        self.redo_attrib = self.element.attrib.copy()

    def undo(self):
        self.element.attrib = self.undo_attrib
        d = self.TE.real_button_drawops
        
        if self.Bstate == "normal":
            if not d.has_key(self.fs_name):
                d[self.fs_name] = {}
            
            d[self.fs_name][self.function] = self.element.attrib.copy()


    def redo(self):
        self.element.attrib = self.redo_attrib
        d = self.TE.real_button_drawops

        if self.Bstate == "normal":
            if not d.has_key(self.fs_name):
                d[self.fs_name] = {}
        
            d[self.fs_name][self.function] = self.element.attrib.copy()

class FrameUndo(object):

    def __init__(self,TE,win_type,frame_element,old_attrib):
        
        self.TE = TE
        
        self.element = frame_element
        focus = self.element.attrib['focus']
        state = self.element.attrib['state']

        self.is_normal_fs = False
        if win_type == "normal" and focus == "yes" and state == "normal":
            self.is_normal_fs = True

        self.element = self.TE.Get_Element(win_type,focus,state)
        
        self.undo_attrib = old_attrib
        self.redo_attrib = self.element.attrib.copy()

    def undo(self):
        self.element.attrib = self.undo_attrib
        if self.is_normal_fs:
            self.TE.normal_frame_style = self.undo_attrib['style']

    def redo(self):
        self.element.attrib = self.redo_attrib
        if self.is_normal_fs:
            self.TE.normal_frame_style = self.redo_attrib['style']

class WindowUndo(object):
    
    def __init__(self,TE,Window_element,old_attrib):
        
        self.TE = TE
        self.element = Window_element

        self.undo_attrib = old_attrib
        self.redo_attrib = Window_element.attrib.copy()

    def undo(self):
        self.TE.set_frame_style_set(self.element,self.undo_attrib['style_set'])

    def redo(self):
        self.TE.set_frame_style_set(self.element,self.redo_attrib['style_set'])
